﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace ScrumPowerTools.Models
{
    internal class ChangesetVisitor
    {
        public event EventHandler<ChangesetVisitEventArgs> ChangesetVisit = delegate { };

        private readonly VersionControlServer versionControlServer;
        private readonly WorkItemVisitor workItemVisitor;
        private Workspace workspace;

        public ChangesetVisitor(WorkItemStore store, VersionControlServer versionControlServer)
        {
            this.versionControlServer = versionControlServer;

            workItemVisitor = new WorkItemVisitor(store);
            workItemVisitor.WorkItemVisit += OnWorkItemVisit;

            try
            {
                workspace = versionControlServer.GetWorkspace(Environment.MachineName, versionControlServer.AuthorizedUser);
            }
            catch
            {
            }
        }

        void OnWorkItemVisit(object sender, WorkItemVisitEventArgs e)
        {
            VisitChangesets(e.WorkItem);
        }

        private void VisitChangesets(WorkItem workItem)
        {
            var externalLinks = workItem.Links.OfType<ExternalLink>();

            foreach (ExternalLink externalLink in externalLinks)
            {
                VisitChangeset(externalLink);
            }
        }

        private void VisitChangeset(ExternalLink externalLink)
        {
            var artifactUri = new Uri(externalLink.LinkedArtifactUri);

            VersionControlArtifactType artifactType = versionControlServer.ArtifactProvider.GetArtifactType(artifactUri);

            if (artifactType == VersionControlArtifactType.Changeset)
            {
                var changeSet = versionControlServer.ArtifactProvider.GetChangeset(artifactUri);

                ChangesetVisit(this, new ChangesetVisitEventArgs(changeSet, ConvertToLocal(changeSet.Changes)));
            }
        }

        private IEnumerable<string> ConvertToLocal(Change[] changes)
        {
            if (workspace == null)
            {
                return new string[0];
            }

            string[] serverItems = changes.Select(c => c.Item.ServerItem).ToArray();
            var localizedServerItems = new List<string>();

            foreach (string serverItem in serverItems)
            {
                string displayItem = null;

                if (workspace != null)
                {
                    displayItem = workspace.TryGetLocalItemForServerItem(serverItem);
                }

                if (string.IsNullOrEmpty(displayItem))
                {
                    displayItem = serverItem;
                }

                localizedServerItems.Add(displayItem);
            }

            return localizedServerItems;
        }

        public void Visit(WorkItem workItem)
        {
            workItemVisitor.Visit(workItem);
        }
    }
}